/******************************************************************************

  Typing Trainer

  tpgt

  Copyright (C) 2004-2007 Sallai Andras
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  
*******************************************************************************/


#include "checkrc.h"
#include <stdio.h>
#include <ncursesw/ncurses.h>
#include "gettext.h"
#include <stdlib.h> /* getenv(); malloc() */
#include <string.h> /* strlen() */

/**** pts ****/
/** @return string, to be free()ed, never NULL */
char *get_rc_filename(void) {
    char *home = (char *) getenv("HOME");
    char *tpgtfilename;
    if (home==NULL) home="/home/user";
    sprintf(tpgtfilename = (char *) malloc(strlen(home)+42),
      "%s/.tpgtrc", home);
    return tpgtfilename;
}

/** This was formerly tcheck() */
void overwrite_tpgtrc(char const *rlang)
{

    FILE *ftpgtrc;
    char *tpgtfilename=get_rc_filename();
    char const*lang;
    char langbuf[64];
    char const *msg=NULL;
    char const*ui_lang="en_US"; /* Dat: default */
    WINDOW *hwindow;

    initscr();
    start_color();
    init_pair(1, COLOR_WHITE, COLOR_BLUE);
    hwindow = newwin(6, 40, 8, 12);
    wbkgd(hwindow, COLOR_PAIR(1));
    box(hwindow, 0, 0);

    if ((ftpgtrc = fopen(tpgtfilename, "w")) == NULL)
    {
# if 0
	clear();
	refresh();
	endwin();
	fprintf(stderr, _("Error opening file: %s\n"), tpgtfilename);
	exit(1);
#endif
        msg=_("Error opening"); 
    } else {
	/**** pts ****/
	/* Dat: see man 7 locale for other env vars and there use order*/
	if      (NULL!=(lang=rlang                ) && lang[0]!='\0' && lang[1]!='\0') memcpy(langbuf,lang,2);
	else if (NULL!=(lang=getenv("LC_ALL"     )) && lang[0]!='\0' && lang[1]!='\0') memcpy(langbuf,lang,2);
	else if (NULL!=(lang=getenv("LC_MESSAGES")) && lang[0]!='\0' && lang[1]!='\0') memcpy(langbuf,lang,2);
	else if (NULL!=(lang=getenv("LANG"       )) && lang[0]!='\0' && lang[1]!='\0') memcpy(langbuf,lang,2);
	else memcpy(langbuf,"en",2); /* Dat: default */
	/* vvv Dat: convert to lower case */
	if (langbuf[0]-'A'+0U<='Z'-'A'+0U) langbuf[0]+='a'-'A';
	if (langbuf[1]-'A'+0U<='Z'-'A'+0U) langbuf[1]+='a'-'A';
	langbuf[2]='\0';

	/* Imp: copy retval of getenv() */
	     if (NULL!=(lang=getenv("LC_ALL"     )) && lang[0]!='\0' && lang[1]!='\0') ui_lang=lang;
	else if (NULL!=(lang=getenv("LC_MESSAGES")) && lang[0]!='\0' && lang[1]!='\0') ui_lang=lang;
	else if (NULL!=(lang=getenv("LANG"       )) && lang[0]!='\0' && lang[1]!='\0') ui_lang=lang;

	fprintf(ftpgtrc, "lang = %s\n", langbuf); /* Dat: keyboard layout being learned */
	fprintf(ftpgtrc, "ui_lang = %s\n", ui_lang); /* Dat: user interface language */
	fflush(ftpgtrc);
	if (ferror(ftpgtrc)) msg=_("Error writing");
	else msg=_("Saved");
	fclose(ftpgtrc);
    }
    
    curs_set(0);    
    mvwaddstr(hwindow, 1, 16, msg);
    mvwaddstr(hwindow, 3, 15, _("[ Ok ]"));
    wrefresh(hwindow);
    wgetch(hwindow);
    curs_set(1);
    delwin(hwindow);
    endwin();

    free(tpgtfilename);
}

/**** pts ****/
/* Dat: `lang =' is the lesson language, i.e. the language whose typing is
 *      being taught.
 * @param key e.g. "lang"
 * @return string, to be free()ed, never NULL
 */
char const*get_value_from_rc(char const *key, char const*default_value) {
    FILE *ftpgtrc;
    char *tpgtfilename=get_rc_filename();
    char *value=NULL;

    if ((ftpgtrc = fopen(tpgtfilename, "r")) == NULL)
    {
#if 0
	clear();
	refresh();
	endwin();
	fprintf(stderr, _("Error opening file: %s\n"), tpgtfilename);
	exit(1);
#endif
    } else {
	/* Read the option file */
        char row[256];
	while (NULL!=fgets(row, 255, ftpgtrc)) { /**** pts ****/
	  char *p=row, *q;
	  row[255]='\0';
	  while (*p!=' ' && *p!='\t' && *p!='\r' && *p!='\n' && *p!='\0') p++;
	  q=p;
	  while (!(*p!=' ' && *p!='\t' && *p!='\r' && *p!='\n' && *p!='\0')) p++;
	  if (*p++!='=') continue;
	  while (!(*p!=' ' && *p!='\t' && *p!='\r' && *p!='\n' && *p!='\0')) p++;
	  if (0==strncmp(row,key,q-row)) {
	    q=p;
	    while (*p!='\r' && *p!='\n' && *p!='\0') p++;
	    if (value==NULL) value=(char*)malloc(sizeof row);
	    memcpy(value,q,p-q); value[p-q]='\0';
	  }
	}
	fclose(ftpgtrc);
    }
    /* fflush(stdout); sleep(2); quit(); */
    free(tpgtfilename);
    return value==NULL ? default_value : (char const*)value;
}

void check_tpgtrc(void)
{
    char const*rlang;
    WINDOW *hwindow;

    initscr();
    start_color();
    init_pair(1, COLOR_WHITE, COLOR_BLUE);
    hwindow = newwin(6, 70, 6, 5);
    wbkgd(hwindow, COLOR_PAIR(1));
    box(hwindow, 0, 0);

    if (NULL==(rlang=get_value_from_rc("lang",NULL))) {
        char *tpgtfilename=get_rc_filename();
	int ch;
	mvwprintw(hwindow, 1, 2, _("I can't find 'lang =' in the %s file!"), tpgtfilename);
	mvwaddstr(hwindow, 2, 2, _("Can I create the file your home?"));
	mvwaddstr(hwindow, 4, 2, _("Press Y or N key!"));
	free(tpgtfilename);
	ch = wgetch(hwindow);
	if (ch-'A'+0U<='Z'-'A'+0U) ch+='a'-'A'; /* Dat: lowercase */
	if (ch == 'y' || ch == 'i') overwrite_tpgtrc(/*rlang:*/NULL);
    } else free((void*)rlang);
    
    delwin(hwindow);
    endwin();
}
