/******************************************************************************

  Typing Trainer

  tpgt

  Copyright (C) 2004-2007 Sallai Andras
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  
*******************************************************************************/

#include <string.h>
#include <ncursesw/ncurses.h>
#include <stdlib.h>
#include <time.h>
#include <wchar.h>

#include "lesson.h"
#include "gettext.h"
#include "checkrc.h"
#include "quit.h"
#include "ustrlen.h"

void help_altgr(void);

void lesson(int lesnum)
{

    FILE *flesson;
    char *filename;
    char *str;
    wchar_t *row; //row[512];
    row = (wchar_t *) malloc(512 * sizeof(wchar_t));
    char const*rlang;
    int i, x, y;
    wchar_t ch; //signed ch;    
    WINDOW *swindow;
    WINDOW *bwindow;
    WINDOW *hwindow;
    WINDOW *ewindow; // End Window
    signed int hiterror = 0;
    signed int rowsnum = 0;
    clock_t timestamp = 0;

    
    
    initscr();
    start_color();
    cbreak();
    /* curs_set(2); */
    curs_set(1);
    
    keypad(stdscr, TRUE);
//    keypad(swindow, TRUE);
//    keypad(hwindow, TRUE);
    
    noecho();
    clear();
    refresh();

    swindow = newwin(12, 60, 2, 8); //Show Window
    bwindow = newwin(14, 63, 1, 7); //BackGround Window
    hwindow = newwin(5, 63, 17, 7); //Help Window   
    
    wrefresh(swindow);
    wrefresh(bwindow);
    wrefresh(hwindow);    

    init_pair(1, COLOR_WHITE, COLOR_BLUE);
    init_pair(2, COLOR_BLACK, COLOR_WHITE);
    wbkgd(swindow, COLOR_PAIR(1));
    wbkgd(bwindow, COLOR_PAIR(1));
    wbkgd(hwindow, COLOR_PAIR(1));

    box(bwindow, 0, 0);
    box(hwindow, 0, 0);    

    wrefresh(swindow);    
    wrefresh(bwindow);

    rlang=get_value_from_rc("lang","en");
    
    // Opening the lesson file 
    filename = (char *) malloc(256 * sizeof(char));
    str = (char *) malloc(64 * sizeof(char));
    sprintf(str, "%02d", lesnum); /* Dat: was numtostr() */

    strcpy(filename, "/usr/share/tpgt/"); 
    strcat(filename, "lessons/"); 
    strcat(filename, rlang);
    strcat(filename, "/");
    strcat(filename, "lesson");
    strcat(filename, str);


    if ((flesson = fopen(filename, "rb")) == NULL)
    {
        strcpy(filename, "../lessons/"); 
        strcat(filename, rlang);
        strcat(filename, "/");
        strcat(filename, "lesson");
        strcat(filename, str);
    }

    if ((flesson = fopen(filename, "rb")) == NULL)
    {
        strcpy(filename, "/usr/local/tpgt/lessons/"); 
        strcat(filename, rlang);
        strcat(filename, "/");
        strcat(filename, "lesson");
        strcat(filename, str);
    }


    if ((flesson = fopen(filename, "rb")) == NULL)
    {
	clear();
	refresh();
	endwin();
	fprintf(stderr, _("Error opening file: %s\n"), filename);
	exit(1);
    }




    /* Help window */
    switch (lesnum)
    {
	case 1 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"F\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"J\" key"));
	    break;
	case 2 :
	    mvwaddstr(hwindow, 1, 2, _("Left middle finger on the \"D\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right middle finger on the \"K\" key"));
	    break;
	case 3 :
	    mvwaddstr(hwindow, 1, 2, _("Left ring finger on the \"S\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right ring finger on the \"L\" key"));
	    break;
	case 4 :
	    mvwaddstr(hwindow, 1, 2, _("Left little finger on the \"A\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right little finger on the \";\" key"));
	    break;
	case 5 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"G\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"H\" key"));
	    break;
	case 6 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"R\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"U\" key"));
	    break;
	case 7 :
	    mvwaddstr(hwindow, 1, 2, _("Left middle finger on the \"E\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right middle finger on the \"I\" key"));
	    break;
	case 8 :
	    mvwaddstr(hwindow, 1, 2, _("Left ring finger on the \"W\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right ring finger on the \"O\" key"));
	    break;
	case 9 :
	    mvwaddstr(hwindow, 1, 2, _("Left little finger on the \"Q\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right little finger on the \"P\" key"));
	    break;
	case 10 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"V\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"M\" key"));
	    break;
	case 11 :
	    mvwaddstr(hwindow, 1, 2, _("Left middle finger on the \"C\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right middle finger on the \",\" key"));
	    break;
	case 12 :
	    mvwaddstr(hwindow, 1, 2, _("Left ring finger on the \"X\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right ring finger on the \".\" key"));
	    break;
	case 13 :
	    mvwaddstr(hwindow, 1, 2, _("Left little finger on the \"Z\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right little finger on the \"/\" key"));
	    break;
	case 14 :
	    mvwaddstr(hwindow, 2, 2, _("Exercises"));
	    break;
	case 15 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"T\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"Y\" key"));
	    break;
	case 16 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"B\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"N\" key"));
	    break;
	case 17 :
	    mvwaddstr(hwindow, 1, 2, _("Left little finger on the \"I'\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right little finger on the \"A'\" key"));
	    break;
	case 18 :
	    mvwaddstr(hwindow, 1, 2, _("Right little finger on the \"O:\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right little finger on the \"U:\" key"));
	    break;
	case 19 :
	    mvwaddstr(hwindow, 1, 2, _("Right little finger on the \"U'\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right little finger on the \"O'\" key"));
	    break;
	case 20 :
	    mvwaddstr(hwindow, 1, 2, _("Right little finger on the O\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right little finger on the U\" key"));
	    break;
	case 21 :
	    mvwaddstr(hwindow, 1, 2, _("Press the \"Shift\" another hand allways!"));
	    mvwaddstr(hwindow, 2, 2, _("Example: Press the \"A\" your left hand and"));
	    mvwaddstr(hwindow, 3, 2, _("press \"Shift\" key right hand."));
	    break;
	case 22 :
	case 23 :
	case 24 :
	case 25 :
	case 26 :
	case 27 :
	case 28 :
	    mvwaddstr(hwindow, 2, 2, _("Exercises"));
	    break;
	case 29 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"4\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"7\" key"));
	    break;
	case 30 :
	    mvwaddstr(hwindow, 1, 2, _("Left middle finger on the \"3\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right middle finger on the \"8\" key"));
	    break;
	case 31 :
	    mvwaddstr(hwindow, 1, 2, _("Left ring finger on the \"2\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right ring finger on the \"9\" key"));
	    break;
	case 32 :
	    mvwaddstr(hwindow, 1, 2, _("Left little finger on the \"1\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Left little finger on the \"0\" key"));
	    break;
	case 33 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"5\" key"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"6\" key"));
	    break;
	case 34 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"!\" key (+Shift with right hand)"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"=\" key (+Shift with left hand"));
	    break;
	case 35 :
	    mvwaddstr(hwindow, 1, 2, _("Left middle finger on the \"+\" key (+Shift with right hand)"));
	    mvwaddstr(hwindow, 2, 2, _("Right middle finger on the \"(\" key (+Shift with left hand"));
	    break;
	case 36 :
	    mvwaddstr(hwindow, 1, 2, _("Left ring finger on the \"\"\" key (+Shift with right hand)"));
	    mvwaddstr(hwindow, 2, 2, _("Right ring finger on the \")\" key (+Shift with left hand"));
	    break;
	case 37 :
	    mvwaddstr(hwindow, 1, 2, _("Left little finger on the \"'\" key (+Shift with right hand)"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"%\" key (+Shift with left hand"));
	    mvwaddstr(hwindow, 3, 2, _("Left little finger on the \"/\" key (+Shift with right hand)"));
	    break;
	case 38 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"[\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 2, 2, _("Left first finger on the \"]\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 3, 2, _("Right little finger on the \"$\" key (+AltGr with right thumb)"));
	    help_altgr();	    
	    break;
	case 39 :
	    mvwaddstr(hwindow, 1, 2, _("Left first finger on the \"@\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 2, 2, _("Left first finger on the \"{\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 3, 2, _("Right first finger on the \"}\" key (+AltGr with right thumb)"));
	    help_altgr();
	    break;
	case 40 :
	    mvwaddstr(hwindow, 1, 2, _("Right little finger on the \"*\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 2, 2, _("Right first finger on the \"`\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 3, 2, _("Left little finger on the \"~\" key (+AltGr with right thumb)"));
	    help_altgr();
	    break;
	case 41 :
	    mvwaddstr(hwindow, 1, 2, _("Left middle finger on the \"^\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 2, 2, _("Left ring finger on the \"#\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 3, 2, _("Left middle finger on the \"&\" key (+AltGr with right thumb)"));
	    help_altgr();
	    break;
	case 42 :
	    mvwaddstr(hwindow, 1, 2, _("Left little finger on the \"<\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 2, 2, _("Left little finger on the \">\" key (+AltGr with right thumb)"));
	    mvwaddstr(hwindow, 3, 2, _("Right middle finger on the \";\" key (+AltGr with right thumb)"));
	    help_altgr();
	    break;

    }
    wrefresh(hwindow);

    /* Runing the lesson */
    while((feof(flesson)) == 0)
    {
	/* read from lesson file */
    	fgetws(row, 255, flesson); 

	/* Cut the last byte */
	row[wcslen(row) - 1] = '\0';

	// Number of row
	rowsnum++;

	/* Show the current row */
	wclear(swindow);
	wrefresh(swindow);
	wattrset(swindow, COLOR_PAIR(2));
	//mvwaddwstr(swindow, 2, 6, row);
	mvwprintw(swindow, 2, 6, "%ls", row); //wchar_t character
        wmove(swindow, 2, 6);    
        wrefresh(swindow);
	wattrset(swindow, COLOR_PAIR(1));
	
	/* Show errors */	
	getyx(swindow, y, x);
	mvwprintw(swindow, 10, 23, "            ");
	mvwprintw(swindow, 10, 23, _("Errors: %d"), hiterror);
	wmove(swindow, y, x);
	wrefresh(swindow);

        /* Show number of row */
	getyx(swindow, y, x);
	mvwprintw(swindow, 10, 12, "          ");
	mvwprintw(swindow, 10, 12, _("Row: %d"), rowsnum);
	wmove(swindow, y, x);
	wrefresh(swindow);

	/* Show number of lesson */
	getyx(swindow, y, x);
	mvwprintw(swindow, 10, 1, "          ");
	mvwprintw(swindow, 10, 1, _("Lesson: %d"), lesnum);
	wmove(swindow, y, x);
	wrefresh(swindow);

        ch = -1; i = 0;
	timestamp = time(NULL);

        while(i+0U < wcslen(row))   
        {

	    //ch = wget_ch(swindow);
	    wget_wch(swindow, &ch);
	    
	    if (ch == 27) break;
	    if (ch == KEY_F(10)) break;
	    if (ch == KEY_F(12)) quit();	    
	    /* 
	       The '&' operation need for example the
	       hungarian accentuated letters! a', o" and u" etc.
	       But by UTF-8 not need
	    */	    
	    while(ch != (row[i] )) //& 255))    
	    {
		    /* Error handling */
		    hiterror++;
		    getyx(swindow, y, x);
		    mvwprintw(swindow, 10, 20, "            ");
		    mvwprintw(swindow, 10, 20, _("Errors: %d"), hiterror);
		    wmove(swindow, y, x);
		    wrefresh(swindow);
		    getyx(swindow, y, x);
		    beep();
		    getyx(swindow, y, x);

		    /* Error hit actual character */
		    mvwaddstr(swindow, 8, 25, "    ");
		    mvwprintw(swindow, 8, 25, " %lc", ch);

		    wmove(swindow, y,x);
		    wrefresh(swindow);
		    if (ch == 27) break;
		    if (ch == KEY_F(10)) break;
		    if (ch == KEY_F(12)) quit();
		    //ch = wgetch(swindow);
		    wget_wch(swindow, &ch);
		    
		    if (ch == 27) break;
		    if (ch == KEY_F(10)) break;
		    if (ch == KEY_F(12)) quit();
	    }/* while */
	    if (ch == 27) break;
	    getyx(swindow, y, x); 
	    wmove(swindow, y + 2, x);	    	    

	    /* which charachter is right, "second row" */
	    //waddch(swindow, ch); 
	    wprintw(swindow,"%lc", ch);

	    wmove(swindow, y, x);
	    wrefresh(swindow);

	    getyx(swindow, y, x);
	    wmove(swindow, y, x + 1);
	    
	    getyx(swindow, y, x);

	    /* Actual charachter */
	    mvwaddstr(swindow, 8, 25, "    ");
	    mvwprintw(swindow, 8, 25, " %lc", ch);

	    wmove(swindow, y,x);

	    ch = -1;
	    i++;	    
	}/* while */
	if (ch == 27 || ch == KEY_F(10)) break;
	ch = wgetch(swindow);
	if (ch == 27 || ch == KEY_F(10)) break;	
	if (ch == KEY_F(12)) quit();
	while( ch != 10)  //Enter pressed
	{
	    beep(); // This isn't the Enter key
	    if (ch == 27 || ch == KEY_F(10)) break;	    
	    ch = wgetch(swindow);
	    if (ch == KEY_F(12)) quit();
	    hiterror++;
	}/* while */
	if (ch == KEY_F(12)) quit();
	if (ch == 27 || ch == KEY_F(10)) break;
    }/* while */
    
    fclose(flesson);



    delwin(swindow);
    delwin(bwindow);    
    delwin(hwindow);


    /***  Result Window ***/
    curs_set(0);
    ewindow = newwin(10, 50, 7, 14); //End Window   
    keypad(ewindow, TRUE);
    wrefresh(ewindow);
    wbkgd(ewindow, COLOR_PAIR(1));
    
    box(ewindow, 0, 0);    
    wrefresh(ewindow);
    mvwaddstr(ewindow, 2, 20, _("Results"));
    mvwprintw(ewindow, 5, 5, _("Errors: %d"), hiterror);
    mvwprintw(ewindow, 7, 22, _("Ok"));
    wrefresh(ewindow);    
    wgetch(ewindow);
    curs_set(1);
    delwin(ewindow);
    endwin();
}





void help_altgr(void)
{
    WINDOW *hwindow;
    curs_set(0);
//    clear();
//    refresh();

    hwindow = newwin(10, 40, 3, 15); //Help Window   
    
    wrefresh(hwindow);    

    init_pair(1, COLOR_WHITE, COLOR_BLUE);
    wbkgd(hwindow, COLOR_PAIR(1));

    box(hwindow, 0, 0);    

    mvwaddstr(hwindow, 3, 5, _("AltGr is right side only."));
    mvwaddstr(hwindow, 5, 3, _("Press with your right thumb!"));    
    mvwaddstr(hwindow, 7, 14, _("[ OK ]"));    
    
    wrefresh(hwindow);
    
    getch();
//    clear();
    refresh();
    delwin(hwindow);
    curs_set(1);
}




